@@ -2836,7 +2836,7 @@ static int trigger_failover_command(int node, const char *command_line,
 							string_append_char(exec_cmd, newmaster->backend_hostname);
 						else
 							/* no valid new master */
-							string_append_char(exec_cmd, "");
+							string_append_char(exec_cmd, "\"\"");
 						break;
 
 					case 'm': /* new master node id */
@@ -2853,7 +2853,7 @@ static int trigger_failover_command(int node, const char *command_line,
 						}
 						else
 							/* no valid new master */
-							string_append_char(exec_cmd, "");
+							string_append_char(exec_cmd, "\"\"");
 						break;
 
 					case 'R': /* new master database directory */
@@ -2862,7 +2862,7 @@ static int trigger_failover_command(int node, const char *command_line,
 							string_append_char(exec_cmd, newmaster->backend_data_directory);
 						else
 							/* no valid new master */
-							string_append_char(exec_cmd, "");
+							string_append_char(exec_cmd, "\"\"");
 						break;
 
 					case 'M': /* old master node id */
