@@ -3,7 +3,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2013	PgPool Global Development Group
+ * Copyright (c) 2003-2014	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -116,6 +116,7 @@ static int pool_hash_insert(POOL_QUERY_HASH *key, POOL_CACHEID *cacheid, bool up
 static uint32 create_hash_key(POOL_QUERY_HASH *key);
 static volatile POOL_HASH_ELEMENT *get_new_hash_element(void);
 static void put_back_hash_element(volatile POOL_HASH_ELEMENT *element);
+static bool is_free_hash_element(void);
 static char *get_relation_without_alias(RangeVar *relation);
 
 /*
@@ -1206,6 +1207,7 @@ int pool_get_database_oid_from_dbname(char *dbname)
 	if (status != POOL_CONTINUE)
 	{    
 		pool_debug("pool_discard_oid_maps_by_db: Failed.");
+		free_select_result(res);
 		return 0;
 	}    
 
@@ -1636,7 +1638,7 @@ static int pool_get_memqcache_blocks(void)
  */
 size_t pool_shared_memory_cache_size(void)
 {
-	int num_blocks;
+	int64 num_blocks;
 	size_t size;
 
 	if (pool_config->memqcache_maxcache > pool_config->memqcache_cache_block_size)
@@ -1650,12 +1652,12 @@ size_t pool_shared_memory_cache_size(void)
 		pool_config->memqcache_cache_block_size;
 	if (num_blocks == 0)
 	{
-		pool_error("pool_shared_memory_cache_size: memqcache_total_size %d should be greater or equal to memqcache_cache_block_size %d",
+		pool_error("pool_shared_memory_cache_size: memqcache_total_size %ld should be greater or equal to memqcache_cache_block_size %d",
 				   pool_config->memqcache_total_size, pool_config->memqcache_cache_block_size);
 		return 0;
 	}
 
-	pool_log("pool_shared_memory_cache_size: number of blocks: %d", num_blocks);
+	pool_log("pool_shared_memory_cache_size: number of blocks: %ld", num_blocks);
 
 	/* Remember # of blocks */
 	pool_set_memqcache_blocks(num_blocks);
@@ -1799,7 +1801,7 @@ static int *pool_fsmm_clock_hand;
  */
 void pool_allocate_fsmm_clock_hand(void)
 {
-	pool_fsmm_clock_hand = pool_shared_memory_create(sizeof(pool_fsmm_clock_hand));
+	pool_fsmm_clock_hand = pool_shared_memory_create(sizeof(*pool_fsmm_clock_hand));
 	*pool_fsmm_clock_hand = 0;
 }
 
@@ -1875,7 +1877,7 @@ static POOL_CACHE_BLOCKID pool_get_block(size_t free_space)
 		return -1;
 	}
 
-	if (free_space < 0 || free_space > POOL_MAX_FREE_SPACE)
+	if (free_space > POOL_MAX_FREE_SPACE)
 	{
 		pool_error("pool_get_block: invalid free space %zd", free_space);
 		return -1;
@@ -1919,13 +1921,13 @@ static void pool_update_fsmm(POOL_CACHE_BLOCKID blockid, size_t free_space)
 		return;
 	}
 
-	if (blockid < 0 || blockid >= pool_get_memqcache_blocks())
+	if (blockid >= pool_get_memqcache_blocks())
 	{
 		pool_error("pool_set_free_space: invalid block id %d", blockid);
 		return;
 	}
 
-	if (free_space < 0 || free_space > POOL_MAX_FREE_SPACE)
+	if (free_space > POOL_MAX_FREE_SPACE)
 	{
 		pool_error("pool_set_free_space: invalid free space %zd", free_space);
 		return;
@@ -2003,6 +2005,16 @@ static POOL_CACHEID *pool_add_item_shmem_cache(POOL_QUERY_HASH *query_hash, char
 	 */
 	pool_init_cache_block(blockid);
 
+	/*
+	 * Make sure that we have at least one free hash element.
+	 */
+	while (!is_free_hash_element())
+	{
+		/* If not, reuse next victim block */
+		blockid = pool_reuse_block();
+		pool_init_cache_block(blockid);
+	}
+
 	/* Get block address on shmem */
 	p = block_address(blockid);
 	bh = (POOL_CACHE_BLOCK_HEADER *)p;
@@ -2298,7 +2310,7 @@ static int pool_delete_item_shmem_cache(POOL_CACHEID *cacheid)
 	pool_debug("pool_delete_item_shmem_cache: cacheid:%d itemid:%d",
 			   cacheid->blockid, cacheid->itemid);
 
-	if (cacheid->blockid < 0 || cacheid->blockid >= pool_get_memqcache_blocks())
+	if (cacheid->blockid >= pool_get_memqcache_blocks())
 	{
 		pool_error("pool_delete_item_shmem_cache: invalid block id %d",
 				   cacheid->blockid);
@@ -2313,7 +2325,7 @@ static int pool_delete_item_shmem_cache(POOL_CACHEID *cacheid)
 		return -1;
 	}
 
-	if (cacheid->itemid < 0 || cacheid->itemid >= bh->num_items)
+	if (cacheid->itemid >= bh->num_items)
 	{
 		/*
 		 * This could happen if the block is reused.  Since contents
@@ -2391,14 +2403,14 @@ static POOL_CACHE_ITEM_HEADER *pool_cache_item_header(POOL_CACHEID *cacheid)
 {
 	POOL_CACHE_BLOCK_HEADER *bh;
 
-	if (cacheid->blockid < 0 || cacheid->blockid >= pool_get_memqcache_blocks())
+	if (cacheid->blockid >= pool_get_memqcache_blocks())
 	{
 		pool_error("pool_cache_item_header: invalid block id %d", cacheid->blockid);
 		return NULL;
 	}
 
 	bh = (POOL_CACHE_BLOCK_HEADER *)block_address(cacheid->blockid);
-	if (cacheid->itemid < 0 || cacheid->itemid >= bh->num_items)
+	if (cacheid->itemid >= bh->num_items)
 	{
 		pool_error("pool_cache_item_header: invalid item id %d", cacheid->itemid);
 		return NULL;
@@ -2415,7 +2427,7 @@ static int pool_init_cache_block(POOL_CACHE_BLOCKID blockid)
 	char *p;
 	POOL_CACHE_BLOCK_HEADER *bh;
 
-	if (blockid < 0 || blockid >= pool_get_memqcache_blocks())
+	if (blockid >= pool_get_memqcache_blocks())
 	{
 		pool_error("pool_init_cache_block: invalid block id %d", blockid);
 		return -1;
@@ -2640,6 +2652,7 @@ POOL_TEMP_QUERY_CACHE *pool_create_temp_query_cache(char *query)
 		if (!p->query)
 		{
 			pool_error("pool_create_temp_query_cache: strdup failed");
+			free(p);
 			return NULL;
 		}
 		p->buffer = pool_create_buffer();
@@ -3116,8 +3129,10 @@ void pool_handle_query_cache(POOL_CONNECTION_POOL *backend, char *query, Node *n
 			{
 				pool_error("ReadyForQuery: pool_commit_cache failed");
 			}
-			free(oids);
-			free(cache_buffer);
+			if (oids)
+				free(oids);
+			if (cache_buffer)
+				free(cache_buffer);
 		}
 		pool_shmem_unlock();
 		POOL_SETMASK(&oldmask);
@@ -3130,7 +3145,7 @@ void pool_handle_query_cache(POOL_CONNECTION_POOL *backend, char *query, Node *n
 	else		/* Non cache safe queries */
 	{
 		/* Non cachable SELECT */
-		if (IsA(node, SelectStmt))
+		if (node && IsA(node, SelectStmt))
 		{
 			if (state == 'I')
 			{
@@ -3709,6 +3724,14 @@ static void put_back_hash_element(volatile POOL_HASH_ELEMENT *element)
 	element->next = elm;
 }
 
+/*
+ * Return true if there's a free hash element.
+ */
+static bool is_free_hash_element(void)
+{
+	return hash_free->next != NULL;
+}
+
 /*
  * Returns shared memory cache stats.
  * Subsequent call to this function will break return value
